package be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl;

import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.*;
import be.iminds.ilabt.jfed.lowlevel.api.AbstractGeniAggregateManager;
import be.iminds.ilabt.jfed.lowlevel.api.ProtoGeniAMExtensions;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.ApiWrapper;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.*;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnectionProvider;
import be.iminds.ilabt.jfed.lowlevel.connection.SfaConnection;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.GeniUrn;

import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;

/**
 * ProtoGeniAMExtensionsWrapper
 */
public class ProtoGeniAMExtensionsWrapper extends ApiWrapper {
    protected final Server amServer;
    protected final ApiInfo.Api api;

    public ProtoGeniAMExtensionsWrapper(Logger logger, GeniUserProvider geniUserProvider, JFedConnectionProvider connectionProvider, JFedPreferences jFedPreferences,
                                        Server amServer, ApiInfo.Api api) {
        super(logger, geniUserProvider, connectionProvider, jFedPreferences);
        assert api != null;
        assert amServer != null;
        this.amServer = amServer;
        this.api = api;
    }


    public boolean isCreateImageSupported() {
        return true;
    }

    public AggregateManagerWrapper.ImageInfo createImage(GeniUrn sliceUrn, List<AnyCredential> sliceCredentials, GeniUrn componentUrn, String imageName,
                                                         boolean global, boolean updatePrepare) throws JFedException {
        GeniUser user = geniUserProvider.getLoggedInGeniUser();
        SfaConnection con = (SfaConnection) connectionProvider.getConnectionByAuthority(user, amServer, api);

        ProtoGeniAMExtensions protoGeniAMExtensions = new ProtoGeniAMExtensions(logger, jFedPreferences);

        AbstractGeniAggregateManager.AggregateManagerReply<ProtoGeniAMExtensions.ImageInfo> reply =
                protoGeniAMExtensions.createImage(
                        con, finalizeSliceCredentials(sliceCredentials), sliceUrn.getValue(), imageName,
                        componentUrn.getValue() /*sliverUrn*/,
                        global/*true*//*global*/,
                        updatePrepare ? true : null,
                        null/*extra options*/);

        if (!reply.getGeniResponseCode().isSuccess())
            throw new JFedException("CreateImage call not successful", reply);

        return reply.getValue();
    }

    public List<? extends AggregateManagerWrapper.ImageInfo> listImages(List<AnyCredential> userCredentials) throws JFedException {
        GeniUser user = geniUserProvider.getLoggedInGeniUser();
        SfaConnection con = (SfaConnection) connectionProvider.getConnectionByAuthority(user, amServer, api);

        ProtoGeniAMExtensions protoGeniAMExtensions = new ProtoGeniAMExtensions(logger, jFedPreferences);

        AbstractGeniAggregateManager.AggregateManagerReply<List<ProtoGeniAMExtensions.ImageInfo>> reply =
                protoGeniAMExtensions.listImages(
                        con, filterCredentials(userCredentials), user.getUserUrnString(), null/*extra options*/);

        if (!reply.getGeniResponseCode().isSuccess())
            throw new JFedException("ListImages call not successful", reply);

        return reply.getValue();
    }

    public boolean deleteImage(List<AnyCredential> userCredentials, GeniUrn imageUrn) throws JFedException {
        GeniUser user = geniUserProvider.getLoggedInGeniUser();
        SfaConnection con = (SfaConnection) connectionProvider.getConnectionByAuthority(user, amServer, api);

        ProtoGeniAMExtensions protoGeniAMExtensions = new ProtoGeniAMExtensions(logger, jFedPreferences);

        AbstractGeniAggregateManager.AggregateManagerReply<Boolean> reply =
                protoGeniAMExtensions.deleteImage(
                        con, filterCredentials(userCredentials), imageUrn.getValue(), user.getUserUrnString(), null/*extra options*/);

        if (!reply.getGeniResponseCode().isSuccess())
            throw new JFedException("CreateImage call not successful", reply);
        if (reply.getValue() == null)
            throw new JFedException("CreateImage did not return success boolean", reply);

        return reply.getValue();
    }

    @Override
    public void setSpeaksFor(List<AnyCredential> speaksForCredentials, @Nullable GeniUrn speakingForUrn) {
        super.setSpeaksFor(speaksForCredentials, speakingForUrn);
        //TODO: This extensionWrapper does not correctly set the extra options for speaks_for
        //TODO: perhaps this needs to be solved by rethinking the entire ExtensionWrapper concept... (and/or by inheriting from AggregateManagerWrapper?)
//
//        //NOTE: speaksFor credential itself should always be supplied as part of argument list of credentials each option takes
//        if (speaksForUserUrn != null)
//            setExtraOptionsForCallsWithCredential(Collections.singletonMap("geni_speaking_for", speaksForUserUrn.getValue()));
    }
}
